#!/usr/bin/Rscript
#--------------------------SCRIPT DESCRIPTION-----------------------------------
#Main goal: prepare US Census sqlite database.  
#This script:
## Create US Census sqlite database by importing csv file with block level data downoladed from NHGIS (https://www.nhgis.org/)
## Extract information (block category (which is used also as id in raster data), state code, county code, GISJOIN) from shapefiles with block boundary. 
### Information extracted from shapefile block files are needed to to join original data from NHGIS csv file with raster data.  
###GRASS GIS is used to preprocessing data, i.e convert from vector to raster. Vector layer are stored in GRASS GIS as sqlite database. 
### Block id in raster data is a block category number in shapefile. We cannot used GISJOIN field as category in raster data because this field is too long. 

#-------------------------END SCRIPT DESCRIPTION--------------------------------

#--------------------------NAMES OF VARIABLES USE IN THIS SCRIPT
#name of database is defined in dasy_parameters.R file

#TABLE NAME IN DATABASE: 
##US_CENSUS - original Census data downloaded from NHGIS website and imported to database
##US_STATES - table contain US states names, abbreviate and number code (i.e )
#BLOCKS_CODE - output from preprocessing of GRASS vector, contain 5 cols: "cat", "STATEFP10","COUNTYFP10", "GISJOIN","COUNTY_CODE"
###COUNTYFP10 is a original county code - always 3 character - ie. 061
###COUNTY_CODE is county code with state - ie. 36061, where last 3 character is county code and 1 or 2 char is state code


#--------------------------START SCRIPT--------------------------

#Import R-script with definition of variables/paths used in calculation 
source("dasy_parameters.R")
pf = getwd()
#Import library 
library(DBI)
library(RSQLite)



#--------------------------CREATE DATABASE--------------------------
dir.create(file.path(us_db_path, fsep = .Platform$file.sep), showWarnings = FALSE, recursive = TRUE)
#Creates an empty database if it does not exist 
us_db <- dbConnect(SQLite(), dbname = file.path(us_db_path,us_db_name, fsep = .Platform$file.sep))

#--------------------------IMPORT DATA FROM CSV TO DB--------------------------

#import csv files into database - create US_CENSUS table. 
#Field types are define based on imported file. NHGIS provide file codebook, which contain "Data Dictionary" - name of variables in file. To use this script with other data please redefine names based on imported file. 
dbWriteTable(conn = us_db, name = "US_CENSUS", value = file.path(pf, us_census_file, fsep = .Platform$file.sep), row.names = FALSE, header = TRUE, field.types=list(GISJOIN="text",YEAR="integer",REGIONA="text",DIVISIONA="text",STATE="text",STATEA="text",COUNTY="text",COUNTYA="text",COUSUBA="text",PLACEA="text",TRACTA="text",BLKGRPA="text",BLOCKA="text",CONCITA="text",RES_ONLYA="text",TRUSTA="text", AIANHHA="text",AITSCEA="text",TTRACTA="text",TBLKGRPA="text",ANRCA="text",CBSAA="text", METDIVA="text", CSAA="text", NECTAA="text",NECTADIVA="text",CNECTAA="text", UAA="text",URBRURALA="text",CDA="text",SLDUA="text",SLDLA="text",ZCTA5A="text",SUBMCDA="text", SDELMA="text", SDSECA="text",SDUNIA="text",NAME="text",SABINSA="text",                                                                                                                  POP="integer",P5POP="integer",NH="integer",NHW="integer",NHB="integer",NHAM="integer",NHAS="integer",NHPI="integer",NHOR="integer",NHTR="integer",H="integer",HW="integer",HB="integer",HAM="integer",HAS="integer",HPI="integer",HOR="integer",HTR="integer"), overwrite=TRUE)
#list field names in table 
dbListFields(us_db, "US_CENSUS")

#Create US_STATES table (name, abbreviate and numeric code)

dbWriteTable(conn = us_db, name = "US_STATES", value =file.path(pf, state_files, fsep = .Platform$file.sep) , row.names = FALSE, header = TRUE,field.types=list(STATE_NAME="text", ABBR="text", STATE_CODE="text"), overwrite=TRUE)
#create list of states based on US_STATES table
list_of_states = dbGetQuery(us_db, "SELECT  ABBR, STATE_CODE from US_STATES")
list_of_states = list_of_states[!list_of_states$ABBR%in%c("AK", "HI", "PR"),]

#List table in DB
#Should be 2 tables US_CENSUS and US_STATES
dbListTables(us_db)

# Close the connection to the database
dbDisconnect(us_db)


#--------------------------PREPROCESS SQLITE VECTOR FILE FROM GRASS--------------------------
#Extract from VECTOR block code (cat number) which is also used as name of block in raster format, GISJOIN, state, county code
#Check if GISJOIN in csv and vector data is the same 

#Block table contain 5 column and will be saved to database: "cns_units", "STATEFP10","COUNTYFP10", "GISJOIN","COUNTY_CODE"
blocks_table = as.data.frame(matrix(ncol=5, nrow=0))

#list state mapset in grass location
states = list.dirs(file.path(grass_data_path, grass_location_name, fsep = .Platform$file.sep), recursive = FALSE, full.names = FALSE)
list_states = states[states%in%tolower(list_of_states$ABBR)]

#Preprocess for each state 
for (state in list_states) {
  
  #construct path to SQLite database with GRASS GIS vector data
  path_to_db=file.path(grass_data_path, grass_location_name, tolower(state),"sqlite/sqlite.db" , fsep = .Platform$file.sep)
  print(path_to_db)
  #connect to database
  db <- dbConnect(SQLite(), dbname = path_to_db) 
  #name of vector layer in GRASS
  state_table_name = paste(state,shp_data_suffix, sep="")
  #select data for state from vector db
  #squery and colnames are defined based on names in shapefile attribute file. To use this script with other data redefine column names based on attribute shapefile.   
  squery = paste("SELECT cat,STATEFP10,COUNTYFP10,TRACTCE10,BLOCKCE10,GISJOIN,COUNTY_CODE from ", state_table_name)
  db_query <- dbGetQuery(db, squery)
  colnames(db_query)=c("cns_units", "STATEFP10","COUNTYFP10","TRACTCE10","BLOCKCE10", "GISJOIN","COUNTY_CODE")
  #create GISJOIN based on vector and check if GISJOIN in vector is the same. GISJOIN is a field provided in NHGIS data, use to join shapefiles and csv data. 
  tmp_code=paste0("G",db_query$STATEFP10, "0",db_query$COUNTYFP10,"0",db_query$TRACTCE10,db_query$BLOCKCE10, sep="")
  tmp=ifelse(tmp_code==db_query$GISJOIN,1,0)
  print(state)
  if (sum(tmp, na.rm = TRUE)!=nrow(db_query)) {next} 
  #add data for state to block table 
  blocks_table=rbind(blocks_table,db_query[,c(1,2,3,6,7)])
  dbDisconnect(db)
}

#--------------------------SAVE BLOCK TABLE WITH BLOCK CODE TO DB--------------------------
#Save data from vector files into BLOCKS_CODE table
#Connet do us_db and save blocks_table 
us_db <- dbConnect(SQLite(), dbname = file.path(us_db_path,us_db_name, fsep = .Platform$file.sep))
#save data frame to database
dbWriteTable(conn = us_db, name = "BLOCKS_CODE", value = blocks_table, row.names = FALSE, overwrite=TRUE)
#disconet database
dbDisconnect(us_db)



