#!/usr/bin/env Rscript 

#--------------------------SCRIPT DESCRIPTION---------------------
#GRASS GIS needs to be instaled to run this script. 
#This script import raster data from GRASS GIS and provide some preprocessing: 
## updata ancillary data based on population data (assign class uninhabited to areas without population)
## select data from US_CENSUS only blocks, that are in census_sp (save it to US_CENSUS_2010 table) 
## calculate areas of each ancillary class in each block and save results to OUT_AREAS table in database


#--------------------------NAMES OF VARIABLES USE IN THIS SCRIPT-
#blocks - data frame for state based on BLOCKS_CODE table 
#census_sp - spatial data for given county with census boundary and ancillary data. 
#blocks_in_county - part of blocks data frame - contain only blocks that are in census_sp [there are only block after rasterization process]
#US_CENSUS_2010 table contain data from US_CENSUS table only for block after rasterization 
#OUT_AREAS - contain areas of ancillary class in each block 


#--------------------------START SCRIPT--------------------------


#Import R-script with definition of variables/paths used in calculation 
source("dasy_parameters.R")
#Import library
library(rgrass7) #work with GRASS GIS 
library(DBI) #work with database 
library(RSQLite) #work with database 

source("list_of_states.R") # in list of states I can define which state should be considered in calculation process. 

#find path to instalation of GRASS GIS 7.0
pf_to_grassgis = system("grass70 --config path", intern = TRUE)

#pf is a path to working directory
pf = getwd()
dir.create(file.path(pf, out_path, fsep = .Platform$file.sep), showWarnings = FALSE, recursive = TRUE)
#initialisation of GRASS GIS location#initialisation of GRASS GIS location
initGRASS(gisBase =pf_to_grassgis, home = tempdir(), gisDbase = grass_data_path, location = grass_location_name, mapset = "PERMANENT", override=TRUE)

#connect to database 
us_db <- dbConnect(SQLite(), dbname = file.path(us_db_path,us_db_name, fsep = .Platform$file.sep))

#--------------------------START CALCULATION FOR STATE--------------------------  
for (state in list_of_states$ABBR) {#start state loop
  #state is an abbreviate- i.e OH
  #return index of state in a vector 
  state_index=which(list_of_states$ABBR==state)  
  #get number_state_code for state
  state_code=list_of_states$STATE_CODE[state_index]
  
  #-------------SELECT DATA FOR STATE FROM BLOCKS_CODE TABLE
  #From BLOCKS_CODE table select data for state, 
  ##STATEFP10 is a colun name define based on shapefile (see r_sqlite.R). To use with other data needs to be changes. 
  squery=paste0("SELECT * FROM BLOCKS_CODE WHERE STATEFP10==","'", state_code, "'" , sep="")
  #data from BLOCKS_CODE table for particular state 
  blocks = dbGetQuery(us_db, squery)
  
  #list of counties in state (by county code)
  list_of_counties=sort(unique(blocks$COUNTY_CODE))
  
  #create directories named state in output directory 
  dir.create(file.path(out_path, state), showWarnings = FALSE)

  #import data for each county from GRASS GIS into spatial data frame.  
  
  #--------------------------START CALCULATION FOR COUNTY--------------------------  
  for (county in list_of_counties) {#start county loop
    
    #--------------------------IMPORT RASTER FROM GRASS GIS TO R--------------------------
    #name (which contain also mapset name) of raster layers in GRASS GIS with block boundary
    cnt_block=paste("cnt_block_",county,"@",tolower(state) , sep="")
    #name of raster layers in GRASS GIS with ancillary data (name contain aslo mapset name)
    #ancillary layer prefix is define in dasy_parameters.R file, i.e. "cnt_anc_2010_3cl"
    cnt_anc=paste(ancillary_layer_prefix,county,"@",tolower(state) , sep="")
    
    print(cnt_block)
    #set region for given county
    ##execGRASS is a function from rgrass7 library, call "g.region" from GRASS GIS
    execGRASS("g.region", raster=cnt_block,flags=c("p"))

    #import county data (boundary, ancillary data) to spatial data frame
    #read raster data from GRASS GIS into R spatial data frame, readRAST is a function from rgrass7 library 
    census_sp=readRAST(vname=c(cnt_block, cnt_anc), useGDAL=TRUE, NODATA=-9999)
    colnames(census_sp@data)=c("cns_units", "ancillary")
    
    #--------------------------ALIGN BLOCK BOUNDARY AND ANCILLARY DATA 
    #both layer have to be in the same extent 
    census_sp@data$ancillary[is.na(census_sp@data$cns_units)] <- NA
    census_sp@data$cns_units[is.na(census_sp@data$ancillary)] <- NA
    
    #-----------LIST OF BLOCKS IN CENSUS SP AND CREATE BLOCK ID (NUMBER FROM 1 TO NB.OF BLOCKS IN COUNTY)
    #list of blocks in census sp
    list_of_census_units=sort(unique(census_sp@data$cns_units))#list of blocks in county
    #create unique census units id from 1 to number of census units in a county
    z=factor(census_sp@data$cns_units,levels=list_of_census_units)
    block_numeration=as.integer(z)
    #assign unique numeration to census sp. Unique numeration is next used to save dasymetric results into spatial data frame 
    census_sp$cns_id=block_numeration
  
    
    #Select only blocks that are in census spatial object (rasterized block)
    blocks_in_county=blocks[blocks$COUNTY_CODE==county & blocks$cns_units%in%list_of_census_units,]
    #sort blocks by cns_units
    blocks_in_county = blocks_in_county[order(blocks_in_county$cns_units),]
    #add block id to blocks_in_county
    blocks_in_county$cns_id=sort(unique(block_numeration))
    rm(block_numeration)
  
    #-----------CREATE US_CENSUS table only for blocks that are in census_sp
    #-----------THIS TABLE WILL BE USED IN ANALYSIS 
    
    #extract from US_CENSUS table blocks for given county, US_CENSUS table was created by importing csv file downloaded from NHGIS
    squery=paste0("SELECT * FROM US_CENSUS WHERE STATEA==","'", state_code, "'", " AND COUNTYA==", "'", unique(blocks_in_county$COUNTYFP10), "'", sep="")
    x=dbGetQuery(us_db, squery)
    
    #porownac x z block_in_county - tj merge and return complete cases - tj. only blocks that are in census sp
    #merge data frame with all blocks for county (data feame named x) and block_in_county data frame (block after rasterization process) 
    #y contain columns from US_CENSUS and BLOCKS_CODE for given county for only rasterized blocks
    #name of variable are based on shapefile attribute table
    y=merge(x=x,y=blocks_in_county, by.y="GISJOIN", all.y=TRUE)
    col_to_remove=c(which(colnames(y)=="P5POP"), which(colnames(y)=="STATEFP10"),which(colnames(y)=="COUNTYFP10"),which(colnames(y)=="COUNTY_CODE"))
    y=y[,-c(col_to_remove)]
    y=y[order(y$cns_id),]

    #-----------UPDATE ANCILLARY DATA BY BLOCK INFORMATION
    #ASSIGN 0 CLASS TO CELLS WHERE BLOCK POPULATION IS 0 
    #extract block id where pop==0
    pop0=y$cns_id[which(y$POP==0)]
  
    #assign 0 to cells where pop is 0
    census_sp$ancillary[census_sp$cns_id%in%pop0]<-0
    
    #-----------CALCULATE AREA OF EACH ANCILLARY CLASS IN BLOCK
    anc_areas = data.frame(cns_id=y$cns_id, state_code=state_code, county_code=unique(y$COUNTYA), GISJOIN=y$GISJOIN)
    for(i in ancillary_class)  {#start area loop
      tmp=ifelse(census_sp@data$ancillary==i,1,0)
      a=as.integer(unname(tapply(tmp,census_sp@data$cns_units,FUN=sum, na.rm=TRUE))) 
      anc_areas = cbind(anc_areas, a)
    }#end area loop
  
    colnames(anc_areas) = c("cns_id", "STATE_CODE", "COUNTY_CODE","GISJOIN", paste("a", ancillary_class, sep=""))
    anc_areas$area=apply(anc_areas[, -c(1:4)], 1, sum)
    
    #-----------SAVE OUTPUTS
    print("save results")
    #save y to database using append option - append data for each county. Create US_CENSUS_2010 table, which contain information only for blocks from spatial data frame (rasterized blocks)
    dbWriteTable(conn=us_db, name="US_CENSUS_2010", value=y, row.names=FALSE, append=TRUE) 
    #save anc_areas to database using append option - append data for each county 
    dbWriteTable(conn=us_db, name="OUT_AREAS", value=anc_areas, row.names=FALSE, append=TRUE) 
  
    rm(tmp,a,anc_areas,x,y,blocks_in_county,pop0)
    print("save rds")
    #save census_sp to rds file
    saveRDS(census_sp, file=file.path(pf, out_path, state, paste0("cnt_", county, ".rds", sep=""), fsep = .Platform$file.sep))
  } #end county loop
  
} #end state loop
dbDisconnect(us_db)    
  

