#!/usr/bin/env Rscript 

#--------------------------SCRIPT DESCRIPTION---------------------
#This script perform dasymetric modeling for each county in each state and save results into spatial data frame census_sp and database tables


#Database table used in this script: 
##US_CENSUS_2010 
##OUT_AREAS 

#Database table created in this script: 
##OUT_DASY_POP 
##OUT_DASY_WEIGHT 

library(DBI)
library(RSQLite)



#dasy_parameters.R contain names and paths to input/output files used during calculation. 
source("dasy_parameters.R") 
source("list_of_states.R") #list of states used in calculation
source("dasy_fk2.R") #funtion to perform dasymetric modeling

#load density_coefficients
load(file.path(us_db_path, "density_coeficients.RData"))

#--------------------------START SCRIPT--------------------------
# Start the clock!
ptm_all <- proc.time()

#connect to database
us_db <- dbConnect(SQLite(), dbname = file.path(us_db_path,us_db_name, fsep = .Platform$file.sep))

#--------------------------START CALCULATION FOR STATE--------------------------  
for (state in list_of_states$ABBR) {#start state loop
  #state is an abbreviate- i.e OH
  #return index of state in a vector 
  state_index=which(list_of_states$ABBR==state)  
  #get number_state_code for state
  state_code=list_of_states$STATE_CODE[state_index]
  
  #create list of counties in state
  squery=paste0("SELECT COUNTY_CODE FROM BLOCKS_CODE WHERE STATEFP10==", "'", state_code, "'", sep="")
  counties=dbGetQuery(us_db, squery)
  list_of_counties = sort(unique(counties$COUNTY_CODE))
  rm(counties)
  
  #--------------------------START CALCULATION FOR COUNTY--------------------------  
  for (county in list_of_counties) {#start county loop
    print(county)
    #substract county code
    county_code=substr(county, nchar(county)-2, nchar(county))
    
    #--------------PREPARE DATA FOR COUNTY----------------
    #select population data for county from US_CENSUS_2010
    squery=paste0("SELECT cns_id,GISJOIN,POP FROM US_CENSUS_2010 WHERE COUNTYA==", "'", county_code, "'", " AND STATEA==","'", state_code, "'", sep="")
    pop_df=dbGetQuery(us_db, squery)
    #select area data for county from OUT_AREAS
    squery=paste0("SELECT * FROM OUT_AREAS WHERE COUNTY_CODE==", "'", county_code, "'"," AND STATE_CODE==",  "'", state_code, "'", sep="")
    area_df=dbGetQuery(us_db, squery)
    
    #read census_sp from rds file
    census_sp = readRDS(file.path(out_path, state, paste0("cnt_", county, ".rds", sep="")))
    #create input_df - data frame with population and areas for county
    input_df=merge(pop_df, area_df, by="GISJOIN")
    rm(pop_df, area_df)
    input_df=input_df[,-4]
    col_names=names(input_df)
    col_names[2]<-"cns_id"
    colnames(input_df)=col_names
    input_df=input_df[order(input_df$cns_id), ]
    
    #--------------CALCULATE WEIGHTS----------------
    weights_df=fk_dasy_weights(density_coefficients,input_df)
    
    #--------------CALCULATE DASYMETRIC POPULATION MODEL----------------
    dasy_df=data.frame(cns_id=input_df$cns_id, GISJOIN=input_df$GISJOIN, state_code=input_df$STATE_CODE, county_code=input_df$COUNTY_CODE, weights_df[,-c(1:4)]*input_df$POP)
    colnames(dasy_df) = c("cns_id", "GISJOIN", "state_code", "county_code", paste("p", ancillary_class, sep=""))
    
    #--------------SAVE RESULTS----------------
    
    #save dasymetric results to spatial data frame (census_sp)
    census_sp@data$pop <-NULL
    census_sp@data$pop=fk_dasy_map(dasy_df, 5 ,census_sp)

    #save spatial data frame (census_sp) as rds file 
    saveRDS(census_sp, file=file.path(out_path, state, paste0("cnt_", county, ".rds", sep=""), fsep = .Platform$file.sep))
    
    #write dasymetric population output into database table ("OUT_DASY_POP")
    dbWriteTable(conn = us_db, name = "OUT_DASY_POP", value=dasy_df , row.names = FALSE, append=TRUE)
    
    #write weights established for population by dasymetric model into database table ("OUT_DASY_WEIGHT")
    dbWriteTable(conn = us_db, name = "OUT_DASY_WEIGHT", value=weights_df , row.names = FALSE, append=TRUE)
    
  } #end county loop
  
}#end state loop


# Stop the clock
proc.time() - ptm_all

